@@ -3,6 +3,7 @@
  *
  * pidgin-sipe
  *
+ * Copyright (C) 2010 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2009 pier11 <pier11@operamail.com>
  *
  *
@@ -22,4 +23,4 @@
  */
 
 SipSecContext
-sip_sec_create_context__krb5(SipSecAuthType type);
+sip_sec_create_context__krb5(guint type);
