@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-11 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <time.h>
 
 #include <glib.h>
@@ -30,6 +34,7 @@
 
 #include "sipe-backend.h"
 #include "sipe-core.h"
+#include "sipe-nls.h"
 
 void sipe_backend_im_message(struct sipe_core_public *sipe_public,
 			     const gchar *from,
@@ -43,6 +48,34 @@ void sipe_backend_im_message(struct sipe_core_public *sipe_public,
 		    time(NULL));
 }
 
+void sipe_backend_im_topic(struct sipe_core_public *sipe_public,
+			   const gchar *with,
+			   const gchar *topic)
+{
+	PurpleAccount *account = sipe_public->backend_private->account;
+	PurpleConversation *conv;
+	gchar *msg;
+
+	/*
+	 * Ensure we have an open conversation with the buddy, otherwise
+	 * message would be lost.
+	 */
+	conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_ANY,
+						     with,
+						     account);
+	if (!conv)
+		conv = purple_conversation_new(PURPLE_CONV_TYPE_IM,
+					       account,
+					       with);
+
+	msg = g_strdup_printf(_("Conversation subject: %s"), topic);
+	sipe_backend_notify_message_info(sipe_public,
+					 (struct sipe_backend_chat_session *)conv,
+					 with,
+					 msg);
+	g_free(msg);
+}
+
 /*
   Local Variables:
   mode: c
