@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-12 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "glib.h"
 #include "glib/gstdio.h"
 #include <fcntl.h>
@@ -29,7 +33,7 @@
 #include "sipe-common.h"
 
 #include "mediamanager.h"
-#include "request.h"
+#include "media-gst.h"
 #include "agent.h"
 
 #include "sipe-backend.h"
@@ -37,9 +41,24 @@
 
 #include "purple-private.h"
 
+/*
+ * GValueArray has been marked deprecated in glib 2.32 but it is still used by
+ * libpurple and libfarsight APIs. Therefore we need to disable the deprecated
+ * warning so that the code still compiles for platforms that enable it.
+ *
+ * Diagnostic #pragma was added in GCC 4.2.0
+ */
+#if defined(__GNUC__) && (__GNUC__ >= 4) && (__GNUC_MINOR__ >= 2)
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
 struct sipe_backend_media {
 	PurpleMedia *m;
 	GSList *streams;
+	/**
+	 * Number of media streams that were not yet locally accepted or rejected.
+	 */
+	guint unconfirmed_streams;
 };
 
 struct sipe_backend_stream {
@@ -48,6 +67,7 @@ struct sipe_backend_stream {
 	gboolean candidates_prepared;
 	gboolean local_on_hold;
 	gboolean remote_on_hold;
+	gboolean accepted;
 };
 
 static void
@@ -72,19 +92,13 @@ on_candidates_prepared_cb(SIPE_UNUSED_PARAMETER PurpleMedia *media,
 			  SIPE_UNUSED_PARAMETER gchar *participant,
 			  struct sipe_media_call *call)
 {
-	struct sipe_backend_media *backend_media = call->backend_private;
 	struct sipe_backend_stream *stream;
-	stream = sipe_backend_media_get_stream_by_id(backend_media, sessionid);
+	stream = sipe_backend_media_get_stream_by_id(call->backend_private, sessionid);
 
 	stream->candidates_prepared = TRUE;
 
-	if (call->candidates_prepared_cb) {
-		GSList *streams = backend_media->streams;
-		for (; streams; streams = streams->next) {
-			struct sipe_backend_stream *s = streams->data;
-			if (!s->candidates_prepared)
-				return;
-		}
+	if (call->candidates_prepared_cb &&
+	    sipe_backend_candidates_prepared(call->backend_private)) {
 		call->candidates_prepared_cb(call, stream);
 	}
 }
@@ -97,8 +111,30 @@ on_state_changed_cb(SIPE_UNUSED_PARAMETER PurpleMedia *media,
 		    struct sipe_media_call *call)
 {
 	SIPE_DEBUG_INFO("sipe_media_state_changed_cb: %d %s %s\n", state, sessionid, participant);
-	if (state == PURPLE_MEDIA_STATE_CONNECTED && call->media_connected_cb)
-		call->media_connected_cb(call);
+	if (state == PURPLE_MEDIA_STATE_END &&
+	    !sessionid && !participant && call->media_end_cb)
+		call->media_end_cb(call);
+}
+
+/* Used externally in purple-plugin.c. This declaration stops the compiler
+ * complaining about missing prototype. */
+void capture_pipeline(gchar *label);
+
+void
+capture_pipeline(gchar *label) {
+	PurpleMediaManager *manager = purple_media_manager_get();
+	GstElement *pipeline = purple_media_manager_get_pipeline(manager);
+	GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS(GST_BIN(pipeline), GST_DEBUG_GRAPH_SHOW_ALL, label);
+}
+
+static void
+on_error_cb(SIPE_UNUSED_PARAMETER PurpleMedia *media, gchar *message,
+	    struct sipe_media_call *call)
+{
+	capture_pipeline("ERROR");
+
+	if (call->error_cb)
+		call->error_cb(call, message);
 }
 
 static void
@@ -109,19 +145,28 @@ on_stream_info_cb(SIPE_UNUSED_PARAMETER PurpleMedia *media,
 		  gboolean local,
 		  struct sipe_media_call *call)
 {
-	struct sipe_backend_media *m = call->backend_private;
-
-	if (type == PURPLE_MEDIA_INFO_ACCEPT && call->call_accept_cb
-	    && !sessionid && !participant)
-		call->call_accept_cb(call, local);
-	else if (type == PURPLE_MEDIA_INFO_HOLD || type == PURPLE_MEDIA_INFO_UNHOLD) {
+	if (type == PURPLE_MEDIA_INFO_ACCEPT) {
+		if (call->call_accept_cb && !sessionid && !participant)
+			call->call_accept_cb(call, local);
+		else if (sessionid && participant) {
+			struct sipe_backend_stream *stream;
+			stream = sipe_backend_media_get_stream_by_id(call->backend_private,
+								     sessionid);
+			if (stream) {
+				if (!stream->accepted && local)
+					 --call->backend_private->unconfirmed_streams;
+				stream->accepted = TRUE;
+			}
+		}
+	} else if (type == PURPLE_MEDIA_INFO_HOLD || type == PURPLE_MEDIA_INFO_UNHOLD) {
 
 		gboolean state = (type == PURPLE_MEDIA_INFO_HOLD);
 
 		if (sessionid) {
 			// Hold specific stream
 			struct sipe_backend_stream *stream;
-			stream = sipe_backend_media_get_stream_by_id(m, sessionid);
+			stream = sipe_backend_media_get_stream_by_id(call->backend_private,
+								     sessionid);
 
 			if (local)
 				stream->local_on_hold = state;
@@ -129,7 +174,7 @@ on_stream_info_cb(SIPE_UNUSED_PARAMETER PurpleMedia *media,
 				stream->remote_on_hold = state;
 		} else {
 			// Hold all streams
-			GSList *i = sipe_backend_media_get_streams(m);
+			GSList *i = sipe_backend_media_get_streams(call->backend_private);
 			for (; i; i = i->next) {
 				struct sipe_backend_stream *stream = i->data;
 
@@ -146,15 +191,19 @@ on_stream_info_cb(SIPE_UNUSED_PARAMETER PurpleMedia *media,
 		if (!sessionid && !participant) {
 			if (type == PURPLE_MEDIA_INFO_HANGUP && call->call_hangup_cb)
 				call->call_hangup_cb(call, local);
-			else if (type == PURPLE_MEDIA_INFO_REJECT && call->call_reject_cb)
+			else if (type == PURPLE_MEDIA_INFO_REJECT && call->call_reject_cb && !local)
 				call->call_reject_cb(call, local);
 		} else if (sessionid && participant) {
 			struct sipe_backend_stream *stream;
-			stream = sipe_backend_media_get_stream_by_id(m, sessionid);
+			stream = sipe_backend_media_get_stream_by_id(call->backend_private,
+								     sessionid);
 
 			if (stream) {
-				m->streams = g_slist_remove(m->streams, stream);
+				call->backend_private->streams = g_slist_remove(call->backend_private->streams, stream);
 				backend_stream_free(stream);
+				if (local && --call->backend_private->unconfirmed_streams == 0 &&
+				    call->call_reject_cb)
+					call->call_reject_cb(call, local);
 			}
 		}
 	}
@@ -169,6 +218,7 @@ sipe_backend_media_new(struct sipe_core_public *sipe_public,
 	struct sipe_backend_media *media = g_new0(struct sipe_backend_media, 1);
 	struct sipe_backend_private *purple_private = sipe_public->backend_private;
 	PurpleMediaManager *manager = purple_media_manager_get();
+	GstElement *pipeline;
 
 	media->m = purple_media_manager_create_media(manager,
 						     purple_private->account,
@@ -179,28 +229,65 @@ sipe_backend_media_new(struct sipe_core_public *sipe_public,
 			 G_CALLBACK(on_candidates_prepared_cb), call);
 	g_signal_connect(G_OBJECT(media->m), "stream-info",
 			 G_CALLBACK(on_stream_info_cb), call);
+	g_signal_connect(G_OBJECT(media->m), "error",
+			 G_CALLBACK(on_error_cb), call);
 	g_signal_connect(G_OBJECT(media->m), "state-changed",
 			 G_CALLBACK(on_state_changed_cb), call);
 
+	/* On error, the pipeline is no longer in PLAYING state and libpurple
+	 * will not switch it back to PLAYING, preventing any more calls until
+	 * application restart. We switch the state ourselves here to negate
+	 * effect of any error in previous call (if any). */
+	pipeline = purple_media_manager_get_pipeline(manager);
+	gst_element_set_state(pipeline, GST_STATE_PLAYING);
+
 	return media;
 }
 
 void
 sipe_backend_media_free(struct sipe_backend_media *media)
 {
-	GSList *stream = media->streams;
-	g_object_unref(media->m);
+	if (media) {
+		GSList *stream = media->streams;
 
-	for (; stream; stream = g_slist_delete_link(stream, stream))
-		backend_stream_free(stream->data);
+		for (; stream; stream = g_slist_delete_link(stream, stream))
+			backend_stream_free(stream->data);
 
-	g_free(media);
+		g_free(media);
+	}
+}
+
+void
+sipe_backend_media_set_cname(struct sipe_backend_media *media, gchar *cname)
+{
+	if (media) {
+		guint num_params = 3;
+		GParameter *params = g_new0(GParameter, num_params);
+		params[0].name = "sdes-cname";
+		g_value_init(&params[0].value, G_TYPE_STRING);
+		g_value_set_string(&params[0].value, cname);
+		params[1].name = "sdes-name";
+		g_value_init(&params[1].value, G_TYPE_STRING);
+		params[2].name = "sdes-tool";
+		g_value_init(&params[2].value, G_TYPE_STRING);
+
+		purple_media_set_params(media->m, num_params, params);
+
+		g_value_unset(&params[0].value);
+		g_free(params);
+	}
 }
 
 #define FS_CODECS_CONF \
 	"# Automatically created by SIPE plugin\n" \
 	"[video/H263]\n" \
-	"farsight-send-profile=videoscale ! ffmpegcolorspace ! fsvideoanyrate ! ffenc_h263 rtp-payload-size=512 ! rtph263pay mtu=512\n";
+	"farsight-send-profile=videoscale ! ffmpegcolorspace ! fsvideoanyrate ! ffenc_h263 rtp-payload-size=30 ! rtph263pay\n" \
+	"\n" \
+	"[audio/PCMA]\n" \
+	"farsight-send-profile=audioconvert ! audioresample ! audioconvert ! alawenc ! rtppcmapay min-ptime=20000000 max-ptime=20000000\n" \
+	"\n" \
+	"[audio/PCMU]\n" \
+	"farsight-send-profile=audioconvert ! audioresample ! audioconvert ! mulawenc ! rtppcmupay min-ptime=20000000 max-ptime=20000000\n";
 
 static void
 ensure_codecs_conf()
@@ -211,21 +298,93 @@ ensure_codecs_conf()
 	if (!g_file_test(filename, G_FILE_TEST_EXISTS)) {
 		int fd = g_open(filename, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
 		gchar *fs_codecs_conf = FS_CODECS_CONF;
-		if (!fd || write(fd, fs_codecs_conf, strlen(fs_codecs_conf)) == -1)
+		if ((fd < 0) || write(fd, fs_codecs_conf, strlen(fs_codecs_conf)) == -1)
 			SIPE_DEBUG_ERROR_NOFORMAT("Can not create fs-codec.conf!");
-		close(fd);
+		if (fd >= 0)
+			close(fd);
 	}
 
 	g_free(filename);
 }
 
+static void
+append_relay(GValueArray *relay_info, const gchar *ip, guint port, gchar *type,
+	     gchar *username, gchar *password)
+{
+	GValue value;
+	GstStructure *gst_relay_info;
+
+	gst_relay_info = gst_structure_new("relay-info",
+			"ip", G_TYPE_STRING, ip,
+			"port", G_TYPE_UINT, port,
+			"relay-type", G_TYPE_STRING, type,
+			"username", G_TYPE_STRING, username,
+			"password", G_TYPE_STRING, password,
+			NULL);
+
+	if (gst_relay_info) {
+		memset(&value, 0, sizeof(GValue));
+		g_value_init(&value, GST_TYPE_STRUCTURE);
+		gst_value_set_structure(&value, gst_relay_info);
+
+		g_value_array_append(relay_info, &value);
+		gst_structure_free(gst_relay_info);
+	}
+}
+
+struct sipe_backend_media_relays *
+sipe_backend_media_relays_convert(GSList *media_relays, gchar *username, gchar *password)
+{
+	GValueArray *relay_info = g_value_array_new(0);
+
+	for (; media_relays; media_relays = media_relays->next) {\
+		struct sipe_media_relay *relay = media_relays->data;
+
+		/* Skip relays where IP could not be resolved. */
+		if (!relay->hostname)
+			continue;
+
+		if (relay->udp_port != 0)
+			append_relay(relay_info, relay->hostname, relay->udp_port,
+				     "udp", username, password);
+
+#ifdef HAVE_ICE_TCP
+		if (relay->tcp_port != 0)
+			append_relay(relay_info, relay->hostname, relay->tcp_port,
+				     "tcp", username, password);
+#endif
+	}
+
+	return (struct sipe_backend_media_relays *)relay_info;
+}
+
+void
+sipe_backend_media_relays_free(struct sipe_backend_media_relays *media_relays)
+{
+	g_value_array_free((GValueArray *)media_relays);
+}
+
+static guint
+stream_demultiplex_cb(const gchar *buf, SIPE_UNUSED_PARAMETER gpointer *user_data)
+{
+	guint8 payload_type = buf[1] & 0x7F;
+	if (payload_type >= 200 && payload_type <=204) {
+		// Looks like RTCP
+		return PURPLE_MEDIA_COMPONENT_RTCP;
+	} else {
+		// Looks like RTP
+		return PURPLE_MEDIA_COMPONENT_RTP;
+	}
+}
+
 struct sipe_backend_stream *
 sipe_backend_media_add_stream(struct sipe_backend_media *media,
 			      const gchar *id,
 			      const gchar *participant,
 			      SipeMediaType type,
-			      gboolean use_nice,
-			      gboolean initiator)
+			      SipeIceVersion ice_version,
+			      gboolean initiator,
+			      struct sipe_backend_media_relays *media_relays)
 {
 	struct sipe_backend_stream *stream = NULL;
 	PurpleMediaSessionType prpl_type = sipe_media_to_purple(type);
@@ -233,14 +392,41 @@ sipe_backend_media_add_stream(struct sipe_backend_media *media,
 	guint params_cnt = 0;
 	gchar *transmitter;
 
-	if (use_nice) {
+	if (ice_version != SIPE_ICE_NO_ICE) {
 		transmitter = "nice";
-		params_cnt = 1;
+		params_cnt = 4;
 
 		params = g_new0(GParameter, params_cnt);
+
 		params[0].name = "compatibility-mode";
 		g_value_init(&params[0].value, G_TYPE_UINT);
-		g_value_set_uint(&params[0].value, NICE_COMPATIBILITY_OC2007R2);
+		g_value_set_uint(&params[0].value,
+				 ice_version == SIPE_ICE_DRAFT_6 ?
+				 NICE_COMPATIBILITY_OC2007 :
+				 NICE_COMPATIBILITY_OC2007R2);
+
+		params[1].name = "transport-protocols";
+		g_value_init(&params[1].value, G_TYPE_UINT);
+#ifdef HAVE_ICE_TCP
+		g_value_set_uint(&params[1].value,
+				 PURPLE_MEDIA_NETWORK_PROTOCOL_UDP |
+				 PURPLE_MEDIA_NETWORK_PROTOCOL_TCP_ACTIVE |
+				 PURPLE_MEDIA_NETWORK_PROTOCOL_TCP_PASSIVE);
+#else
+		g_value_set_uint(&params[1].value,
+				 PURPLE_MEDIA_NETWORK_PROTOCOL_UDP);
+#endif
+
+		params[2].name = "demultiplex-func";
+		g_value_init(&params[2].value, G_TYPE_POINTER);
+		g_value_set_pointer(&params[2].value, stream_demultiplex_cb);
+
+		if (media_relays) {
+			params[3].name = "relay-info";
+			g_value_init(&params[3].value, G_TYPE_VALUE_ARRAY);
+			g_value_set_boxed(&params[3].value, media_relays);
+		} else
+			--params_cnt;
 	} else {
 		// TODO: session naming here, Communicator needs audio/video
 		transmitter = "rawudp";
@@ -250,15 +436,21 @@ sipe_backend_media_add_stream(struct sipe_backend_media *media,
 	ensure_codecs_conf();
 
 	if (purple_media_add_stream(media->m, id, participant, prpl_type,
-				    initiator, transmitter, params_cnt, params)) {
+				    initiator, transmitter, params_cnt,
+				    params)) {
 		stream = g_new0(struct sipe_backend_stream, 1);
 		stream->sessionid = g_strdup(id);
 		stream->participant = g_strdup(participant);
 		stream->candidates_prepared = FALSE;
 
 		media->streams = g_slist_append(media->streams, stream);
+		if (!initiator)
+			++media->unconfirmed_streams;
 	}
 
+	if (params && media_relays)
+		g_value_unset(&params[3].value);
+
 	g_free(params);
 
 	return stream;
@@ -298,8 +490,28 @@ sipe_backend_media_add_remote_candidates(struct sipe_backend_media *media,
 					 struct sipe_backend_stream *stream,
 					 GList *candidates)
 {
+	GList *udp_candidates = NULL;
+
+#ifndef HAVE_ICE_TCP
+	while (candidates) {
+		PurpleMediaCandidate *candidate = candidates->data;
+		PurpleMediaNetworkProtocol proto;
+
+		proto = purple_media_candidate_get_protocol(candidate);
+		if (proto == PURPLE_MEDIA_NETWORK_PROTOCOL_UDP)
+			udp_candidates = g_list_append(udp_candidates, candidate);
+
+		candidates = candidates->next;
+	}
+
+	candidates = udp_candidates;
+#endif
+
+
 	purple_media_add_remote_candidates(media->m, stream->sessionid,
 					   stream->participant, candidates);
+
+	g_list_free(udp_candidates);
 }
 
 gboolean sipe_backend_media_is_initiator(struct sipe_backend_media *media,
@@ -315,6 +527,18 @@ gboolean sipe_backend_media_accepted(struct sipe_backend_media *media)
 	return purple_media_accepted(media->m, NULL, NULL);
 }
 
+gboolean
+sipe_backend_candidates_prepared(struct sipe_backend_media *media)
+{
+	GSList *streams = media->streams;
+	for (; streams; streams = streams->next) {
+		struct sipe_backend_stream *s = streams->data;
+		if (!s->candidates_prepared)
+			return FALSE;
+	}
+	return TRUE;
+}
+
 GList *
 sipe_backend_media_get_active_local_candidates(struct sipe_backend_media *media,
 					       struct sipe_backend_stream *stream)
@@ -423,23 +647,34 @@ sipe_backend_set_remote_codecs(struct sipe_backend_media *media,
 }
 
 GList*
-sipe_backend_get_local_codecs(struct sipe_media_call *call,
+sipe_backend_get_local_codecs(struct sipe_backend_media *media,
 			      struct sipe_backend_stream *stream)
 {
-	GList *codecs = purple_media_get_codecs(call->backend_private->m,
+	GList *codecs = purple_media_get_codecs(media->m,
 						stream->sessionid);
 	GList *i = codecs;
+	gboolean is_conference = (g_strstr_len(stream->participant,
+					       strlen(stream->participant),
+					       "app:conf:audio-video:") != NULL);
 
 	/*
 	 * Do not announce Theora. Its optional parameters are too long,
 	 * Communicator rejects such SDP message and does not support the codec
 	 * anyway.
+	 *
+	 * For some yet unknown reason, A/V conferencing server does not accept
+	 * voice stream sent by SIPE when SIREN codec is in use. Nevertheless,
+	 * we are able to decode incoming SIREN from server and with MSOC
+	 * client, bidirectional call using the codec works. Until resolved,
+	 * do not try to negotiate SIREN usage when conferencing. PCMA or PCMU
+	 * seems to work properly in this scenario.
 	 */
 	while (i) {
 		PurpleMediaCodec *codec = i->data;
 		gchar *encoding_name = purple_media_codec_get_encoding_name(codec);
 
-		if (sipe_strequal(encoding_name,"THEORA")) {
+		if (sipe_strequal(encoding_name,"THEORA") ||
+		    (is_conference && sipe_strequal(encoding_name,"SIREN"))) {
 			GList *tmp;
 			g_object_unref(codec);
 			tmp = i->next;
@@ -458,15 +693,27 @@ struct sipe_backend_candidate *
 sipe_backend_candidate_new(const gchar *foundation,
 			   SipeComponentType component,
 			   SipeCandidateType type, SipeNetworkProtocol proto,
-			   const gchar *ip, guint port)
+			   const gchar *ip, guint port,
+			   const gchar *username,
+			   const gchar *password)
 {
-	return (struct sipe_backend_candidate *)purple_media_candidate_new(
-		foundation,
+	PurpleMediaCandidate *c = purple_media_candidate_new(
+		/* Libnice and Farsight rely on non-NULL foundation to
+		 * distinguish between candidates of a component. When NULL
+		 * foundation is passed (ie. ICE draft 6 does not use foudation),
+		 * use username instead. If no foundation is provided, Farsight
+		 * may signal an active candidate different from the one actually
+		 * in use. See Farsight's agent_new_selected_pair() in
+		 * fs-nice-stream-transmitter.h where first candidate in the
+		 * remote list is always selected when no foundation. */
+		foundation ? foundation : username,
 		component,
 		sipe_candidate_type_to_purple(type),
 		sipe_network_protocol_to_purple(proto),
 		ip,
 		port);
+	g_object_set(c, "username", username, "password", password, NULL);
+	return (struct sipe_backend_candidate *)c;
 }
 
 void
@@ -557,14 +804,6 @@ sipe_backend_candidate_get_protocol(struct sipe_backend_candidate *candidate)
 	return purple_network_protocol_to_sipe(proto);
 }
 
-void
-sipe_backend_candidate_set_username_and_pwd(struct sipe_backend_candidate *candidate,
-					    const gchar *username,
-					    const gchar *password)
-{
-	g_object_set(candidate, "username", username, "password", password, NULL);
-}
-
 static void
 remove_lone_candidate_cb(SIPE_UNUSED_PARAMETER gpointer key,
 			 gpointer value,
@@ -626,22 +865,25 @@ sipe_backend_get_local_candidates(struct sipe_backend_media *media,
 void
 sipe_backend_media_accept(struct sipe_backend_media *media, gboolean local)
 {
-	purple_media_stream_info(media->m, PURPLE_MEDIA_INFO_ACCEPT,
-				 NULL, NULL, local);
+	if (media)
+		purple_media_stream_info(media->m, PURPLE_MEDIA_INFO_ACCEPT,
+					 NULL, NULL, local);
 }
 
 void
 sipe_backend_media_hangup(struct sipe_backend_media *media, gboolean local)
 {
-	purple_media_stream_info(media->m, PURPLE_MEDIA_INFO_HANGUP,
-				 NULL, NULL, local);
+	if (media)
+		purple_media_stream_info(media->m, PURPLE_MEDIA_INFO_HANGUP,
+					 NULL, NULL, local);
 }
 
 void
 sipe_backend_media_reject(struct sipe_backend_media *media, gboolean local)
 {
-	purple_media_stream_info(media->m, PURPLE_MEDIA_INFO_REJECT,
-				 NULL, NULL, local);
+	if (media)
+		purple_media_stream_info(media->m, PURPLE_MEDIA_INFO_REJECT,
+					 NULL, NULL, local);
 }
 
 static PurpleMediaSessionType sipe_media_to_purple(SipeMediaType type)
@@ -690,9 +932,20 @@ static PurpleMediaNetworkProtocol
 sipe_network_protocol_to_purple(SipeNetworkProtocol proto)
 {
 	switch (proto) {
-		case SIPE_NETWORK_PROTOCOL_TCP:	return PURPLE_MEDIA_NETWORK_PROTOCOL_TCP;
-		case SIPE_NETWORK_PROTOCOL_UDP:	return PURPLE_MEDIA_NETWORK_PROTOCOL_UDP;
-		default:						return PURPLE_MEDIA_NETWORK_PROTOCOL_TCP;
+#ifdef HAVE_ICE_TCP
+		case SIPE_NETWORK_PROTOCOL_TCP_ACTIVE:
+			return PURPLE_MEDIA_NETWORK_PROTOCOL_TCP_ACTIVE;
+		case SIPE_NETWORK_PROTOCOL_TCP_PASSIVE:
+			return PURPLE_MEDIA_NETWORK_PROTOCOL_TCP_PASSIVE;
+#else
+		case SIPE_NETWORK_PROTOCOL_TCP_ACTIVE:
+		case SIPE_NETWORK_PROTOCOL_TCP_PASSIVE:
+			return PURPLE_MEDIA_NETWORK_PROTOCOL_TCP;
+#endif
+		case SIPE_NETWORK_PROTOCOL_UDP:
+			return PURPLE_MEDIA_NETWORK_PROTOCOL_UDP;
+		default:
+			return PURPLE_MEDIA_NETWORK_PROTOCOL_UDP;
 	}
 }
 
@@ -700,9 +953,19 @@ static SipeNetworkProtocol
 purple_network_protocol_to_sipe(PurpleMediaNetworkProtocol proto)
 {
 	switch (proto) {
-		case PURPLE_MEDIA_NETWORK_PROTOCOL_TCP: return SIPE_NETWORK_PROTOCOL_TCP;
-		case PURPLE_MEDIA_NETWORK_PROTOCOL_UDP: return SIPE_NETWORK_PROTOCOL_UDP;
-		default:								return SIPE_NETWORK_PROTOCOL_UDP;
+#ifdef HAVE_ICE_TCP
+		case PURPLE_MEDIA_NETWORK_PROTOCOL_TCP_ACTIVE:
+			return SIPE_NETWORK_PROTOCOL_TCP_ACTIVE;
+		case PURPLE_MEDIA_NETWORK_PROTOCOL_TCP_PASSIVE:
+			return SIPE_NETWORK_PROTOCOL_TCP_PASSIVE;
+#else
+		case PURPLE_MEDIA_NETWORK_PROTOCOL_TCP:
+			return SIPE_NETWORK_PROTOCOL_TCP_ACTIVE;
+#endif
+		case PURPLE_MEDIA_NETWORK_PROTOCOL_UDP:
+			return SIPE_NETWORK_PROTOCOL_UDP;
+		default:
+			return SIPE_NETWORK_PROTOCOL_UDP;
 	}
 }
 
